
require "import"
import "android.content.*"
import "android.net.Uri"
import "android.speech.*"
import "android.widget.*"
import "android.os.Handler"
import "android.os.Looper"
import "java.net.URLEncoder"
import "cjson"
import "android.media.MediaPlayer"

local konteks = this or service

-- Memastikan koneksi jaringan aktif
local function isConnected()
    local cm = konteks.getSystemService(Context.CONNECTIVITY_SERVICE)
    local activeNetwork = cm and cm.getActiveNetworkInfo()
    return activeNetwork ~= nil and activeNetwork.isConnectedOrConnecting()
end

-- Memuat konfigurasi dari SharedPreferences
local function loadConfig()
    local prefs = konteks.getSharedPreferences("appConfig", Context.MODE_PRIVATE)
    return {
        vibrate = prefs.getBoolean("vibrate", true),
        correction = prefs.getBoolean("correction", true),
        translate = prefs.getBoolean("translate", false),
        sound = prefs.getBoolean("sound", true),
        autoSend = prefs.getBoolean("autoSend", false),
        tts = prefs.getBoolean("tts", false), -- Load TTS configuration
        sourceLang = prefs.getString("sourceLang", "id"),
        translateLang = prefs.getString("translateLang", "en"),
        aiModel = prefs.getString("aiModel", "openai"),
        correctionMode = prefs.getString("correctionMode", "punctuation"),
        textEnding = prefs.getString("textEnding", "nothing"),
        customInstruction = prefs.getString("customInstruction", ""),
        customSendText = prefs.getString("customSendText", "Kirim")
    }
end

-- Memutar teks dengan Text-to-Speech menggunakan MediaPlayer
local function playTTS(text)
    local ttsUrl = "https://api.siputzx.my.id/api/tools/ttsgoogle?text=" .. URLEncoder.encode(text, "UTF-8")
    
    if not isConnected() then
        showToast("Tidak ada koneksi jaringan.")
        return
    end
    
    local player = MediaPlayer()
    player.setOnErrorListener(function(mp, what, extra)
        showToast("Error memutar audio: " .. what)
        mp.reset()
    end)
    
    pcall(function()
        player.reset()
        player.setDataSource(ttsUrl)
        player.setOnPreparedListener(function(mp)
            mp.start() -- Mulai memutar audio setelah media dipersiapkan
        end)
        player.prepareAsync() -- Menggunakan prepareAsync untuk streaming audio
    end)
end

-- Membersihkan akhiran teks
local function cleanTextEnding(text)
    return text:gsub("[%s\n]+$", "")
end

-- Menambahkan akhiran teks sesuai pengaturan
local function appendTextEnding(text, textEnding)
    text = cleanTextEnding(text)
    if textEnding == "new line" then
        return text .. "\n"
    elseif textEnding == "space" then
        return text .. " "
    elseif textEnding == "dot" then
        return text .. "."
    else
        return text
    end
end

-- Menangani teks yang telah diproses
local function handleProcessedText(text)
    local config = loadConfig()
    text = appendTextEnding(text, config.textEnding)
    
    if config.tts then
        playTTS(text) -- Jika TTS aktif, putar suara
    else
        service.paste(text) -- Jika TTS tidak aktif, tempelkan teks
    end
    
    if config.autoSend then
        autoSend() -- Kirim teks jika auto-send diaktifkan
    end
end

-- Proses teks dengan API jika perlu koreksi
local function processTextWithAPI(text, callback)
    local config = loadConfig()
    local apiUrl = "https://www.velyn.biz.id/api/ai/openai?prompt=" .. URLEncoder.encode(text, "UTF-8")

    if config.correction then
        Http.get(apiUrl, {}, function(status, data)
            if status == 200 then
                local success, response = pcall(function() return cjson.decode(data) end)
                if success and response and response.data then
                    callback(nil, response.data)
else
                    callback("Gagal memproses API.", nil)
                end
            else
                callback("Gagal mengakses API.", nil)
            end
        end)
    else
        callback(nil, text)
    end
end

-- Menangani pengenalan suara dan koreksi
local function startListening()
    local config = loadConfig()
    local speechRecognizer = SpeechRecognizer.createSpeechRecognizer(konteks)
    local speechListener = RecognitionListener{
        onResults = function(results)
            local data = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
            if data and data.size() > 0 then
                local originalText = data.get(0)
                processTextWithAPI(originalText, function(error, processedText)
                    if error then
                        handleProcessedText(originalText)
                    else
                        handleProcessedText(processedText)
                    end
                end)
            else
                service.paste("")
            end
            speechRecognizer.destroy()
        end
    }

    local recognizerIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, Locale.getDefault())
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_PARTIAL_RESULTS, true)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_MAX_RESULTS, 3)

    speechRecognizer.setRecognitionListener(speechListener)
    speechRecognizer.startListening(recognizerIntent)
end

task(1, function()
    startListening()
end)

return true

